#region Preset using directives

using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;

using VirtualLab.Programming;
using VirtualLabAPI.Core.BasicFunctions;
using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.DataVisualization;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Functions;
using VirtualLabAPI.Core.GeometryDescription;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.Core.Materials;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.Numerics.Region2D;
using VirtualLabAPI.Core.OpticalSystems;
using VirtualLabAPI.Core.Propagation;
using VirtualLabAPI.Core.SupportFunctions;
using Wyrowski.Infrastructure.Support;
using VirtualLabAPI.Core.ExportImport;
using VirtualLabWrapperAPI.Wrapper;

#endregion

namespace OwnCode {
	public class VLModule {
		//define file name of optical system to simulate
		string fileNameOs = @"D:\Data\Focusing Ideal Lens\OS_FocusingIdealLens_HighNA.os";

		//define name of parameter #1 to modify
		string nameParameter1 = "_planeWaveLightSource.AchromaticValue.X";
		//define ID of element for parameter #1 to modify
		int IDParameter1 = 0;
		//define list of values
		double[] valuesForParameter1 = new double[]{MathFunctions.DegreeToRadian(10),
													MathFunctions.DegreeToRadian(20),
													MathFunctions.DegreeToRadian(30),
													MathFunctions.DegreeToRadian(40),
													MathFunctions.DegreeToRadian(50)};


		//define name of detector to use (can be only part of the full detector name)
		string nameDetectorToUse = "600";
		//define name of sub-detector to use (can be only part of the full detector name)
		//string nameSubDetectorToUse = "Size X";
		string nameSubDetectorToUse = "Original Data";


		//run method of the module
		public void Run() {
			//load optical setup
			Lightpath osToUseForSimulation = Lightpath.Load(fileNameOs);
			//error handling
			if (osToUseForSimulation == null) {
				Globals.DataDisplay.LogError("Optical Setup could not be loaded");
				return;
			}

			//define list for numerical results
			List<PhysicalValue> listNumericalValuesFromSimulations = new List<PhysicalValue>();

			//loop over all parameters to modify
			for (int runAllParameters = 0; runAllParameters < valuesForParameter1.Length; runAllParameters++) {
				//set current parameter to load optical setup
				LightPathExportImportSupport.ChangeParameterFromExtern(ref osToUseForSimulation,
																	   this.IDParameter1,
																	   nameParameter1,
																	   valuesForParameter1[runAllParameters]);


				//generate simulation wrapper
				LightPathDiagramWrapper osWrapper = new LightPathDiagramWrapper(osToUseForSimulation, false);
				//perform simuation
				osWrapper.Perform();

				//get results from result matrix
				List<DetectorResultObject> simulationResults = osWrapper.SimulationResults;

				//define list of results that match to the detector name
				List<DetectorResultObject> simulationResult_MatchDetector = new List<DetectorResultObject>();
				//loop run through all simulation results
				for (int runSimulationResults = 0; runSimulationResults < simulationResults.Count; runSimulationResults++) {
					//check whether detector name matches (is part of) the name of current result 
					if (simulationResults[runSimulationResults].Description.Contains(nameDetectorToUse)) {
						//add detector result to list
						simulationResult_MatchDetector.Add(simulationResults[runSimulationResults]);
					}
				}

				//error handling (if no result match with detector name)
				if (simulationResult_MatchDetector.Count == 0) {
					Globals.DataDisplay.LogError("No detector result that match with the detector string found.");
					return;
				}

				//define boolean whether sub detector was found
				bool subdetectorFound = false;
				//loop over all results
				for (int runSimulationResults_SubDetector = 0; runSimulationResults_SubDetector < simulationResult_MatchDetector.Count; runSimulationResults_SubDetector++) {
					//get current simulation result
					DetectorResultObject currDetResult = simulationResult_MatchDetector[runSimulationResults_SubDetector];
					//check type of data
					if (currDetResult.Data is IDocument) {
						//in case of IDocument we show the result directly in VLF main window
						Globals.DataDisplay.ShowDocument((currDetResult.Data as IDocument), currDetResult.Description + " ==> Module Simulation #" + (runAllParameters + 1).ToString());
						subdetectorFound = true;
					}
					else if (currDetResult.Data is List<PhysicalValue>) {
						//get list of physical value
						List<PhysicalValue> listDetResult = (currDetResult.Data as List<PhysicalValue>);
						//loop over all detector results
						for (int runSubDetectorResults = 0; runSubDetectorResults < listDetResult.Count; runSubDetectorResults++) {
							if (listDetResult[runSubDetectorResults].Comment.Contains(this.nameSubDetectorToUse)) {
								listNumericalValuesFromSimulations.Add(listDetResult[runSubDetectorResults]);
								subdetectorFound = true;
								break;
							}
						}
					}

					if (subdetectorFound) {
						break;
					}
				}
				//plot message that simulation of current parameter set is finished
				Globals.DataDisplay.LogMessage("Simulation of parameter value #" + (runAllParameters + 1).ToString() + " finished.");
			}

			//check whether numerical values are present
			if (listNumericalValuesFromSimulations.Count > 0) {
				//check whether more than one result is present
				if (listNumericalValuesFromSimulations.Count == 1) {
					//log detector result to detector window in VLF main window
					Globals.DataDisplay.LogDetectorResult("Results from Module Execution", listNumericalValuesFromSimulations);
				}
				else {
					//if more than one numerical value given => generate data array

					//generate field for values and coordinates
					CFieldDerivative1DReal cfMeasuredValues = new CFieldDerivative1DReal(this.valuesForParameter1.Length);
					CFieldDerivative1DReal cfCoordinates = new CFieldDerivative1DReal(this.valuesForParameter1.Length);
					//fill data
					for (int runData = 0; runData < this.valuesForParameter1.Length; runData++) {
						//store coordinate
						cfCoordinates[runData] = this.valuesForParameter1[runData];
						//store measured value
						cfMeasuredValues[runData] = listNumericalValuesFromSimulations[runData].Value;
					}

					//generate data array
					DataArray1D daNumericalValuesPerSimulation = new DataArray1D(new ComplexField1DArray(new ComplexField[] { cfMeasuredValues }),                      //data matrix for measured data
																				 new PhysicalProperty[] { listNumericalValuesFromSimulations[0].PhysicalProperty },     //physical property of measured data
																				 new string[] { listNumericalValuesFromSimulations[0].Comment },                        //name of measured data
																				 cfCoordinates,                                                                         //coordinate information
																				 PhysicalProperty.AngleDeg,                                                             //physical property of coordinates
																				 "Incident Angle",                                                                      //name of coordinates
																				 this.valuesForParameter1[this.valuesForParameter1.Length - 1] + 1e-5);                 //non equisitant coordinates ==> require "upper" limit of last datapoint
																																										//show generated data array
					Globals.DataDisplay.ShowDocument(daNumericalValuesPerSimulation, "Numerical Result Generated by Simulation Module");
				}
			}
		}
	}
}
